**************** COMET UDFs - written by Mike Reed, CompuSolve ** 06/28/90 ****
*                                                                             *
* This file contains a collection of functions to aid in the                  *
* development of automated background communications using COMET.  All        *
* commands are dBASEIII compatible to avoid having to write different         *
* source code for each dBASE dialect, except for:
*
*    1) FUNCTION statements - dBASE and any other dialects that don't support *
*       it, should simply replace "FUNCTION" stmts with "PROCEDURE"s.             *
*
*    2) _ONDCD(), _ONDONE() & _ONTIME() -  you'll need to select the proper                                      *
*        key event trapping command for your specific dialect.  These         *
*        functions come set for FoxPro as they use ON KEY LABEL xx commands.  *
*        dBASE and FoxBase equivalent statements are commented out in each    *
*        of these three functions.
*
*******************************************************************************


*                   Summary of Functions                                      *
*                 ------------------------                                    *
                                                                              *
*   _AUTOANS(port#:N, passin:C, passout:C, timeout:N, hangup:L, cmds:C)

*   _CALL(port#:N, tel#:C, passin:C, passout:C, timeout:N , hangup:L, cmds:C)

*   _CLOSE(port#:N)

*   _COMET(command:C)

*   _DISCON(port#:N)

*   _FIND(port#, string:C, timeout:N)

*   _FLUSH(port#:N)

*   _INPCHR(port#:N)

*   _INPSTR(port#:N, #chars:N)

*   _INPWAIT(port#:N, #chars:N, timeout:N)

*   _ISCTS(port#:N)

*   _ISDCD(port#:N)

*   _ISDSR(port#:N)

*   _MODEM()

*   _ISRI(port#:N)

*   _ONDCD(port#:N, procedure:C)

*   _ONDONE(port#:N, procedure:C)

*   _ONTIME(port#:N, seconds:N, procedure:C)

*   _OPEN(port#:N, baud:N, parity:C, data:N, stop:N, flow:C)

*   _OUTPUT(port#:N, string:C)

*   _RXCNT(port#:N)

*   _SETDTR(port#:N, on/off:L)

*   _SETRTS(port#:N, on/off:L)
*********************************************************************


****************************** _AUTOANS *******************************
* Purpose:  a) put Hayes compatible modem into the auto-answer mode
*           b) upon modem answering a call, jump to a procedure that does
*              password checking (optional) and starts up any file transfer(s)
*              (also optional).
*
* SYNTAX: _AUTOANS(portN, passinC, passoutC, timeoutN, hangupL, cmdsC)

* Parameters:
*   portN - specifies COM port  (Will be used in multi port version, COMET-MP)
*   passinC - password expected from calling modem (none if passinC = "")
*   passoutC - password sent out to calling modem (none if passoutC = "")
*   timeoutC - # seconds to wait for valid password receipt before aborting.
*   hangupL  - indicates if auto-disconnect should occur upon completion of cmds
*   cmdsC    - a list of COMET library command(s) delimited by semicolons.
*              ie. cmdsC = "YSEND \ADIR\*.* ; YRECV"  (2 commands)

* Returns:
*   <expL> indicating success (.T.) or failure (.F)  of auto-answer setup.
*
*   NOTE: The COM port must be _OPEN() before using this function!


FUNCTION _AUTOANS
PARAMETERS portN, passinC, passoutC, timeoutN, hangupL, cmdsC
PRIVATE Cmd, Cr
Cr = CHR(13)                                && for hayes modem cmds - end with CR

* Test if a COM port is OPEN
IF TYPE("_Port_") + TYPE("_Error_") # "NC"  && _OPEN() should have declared both
    RETURN .F.                              && If not, can't go on - we,ve failed
ENDIF

IF .NOT. _ISDCD(_Port_)                     && Test if carrier already there (direct connect)
 Cmd = _FLUSH(_Port_)                        && clear buffer
 Cmd = _COMET('OUTPUT AT E0 C1 S0=1 S0?' + Cr)  && Set echo off, ring count=1 & verify

 IF .NOT. _FIND(_Port_, 'OK', 2)           && Check for modem OK
    _Error_ = "_AUTOANS() failed - no or invalid modem setup reply"
    RETURN .F.                              && If no/invalid reply, failed
 ENDIF

* Modem now set for auto-answer on 1st ring

* Trap for detection of DCD signal = modem answered & connected
 ONStat = _ONDCD(_Port_, '__CONNECT')
 IF ONStat
    PUBLIC _PwdIn_, _PwdOut_, _Cmds_, CmdQ    &&  Declare for __Connect's usage
    PUBLIC _HangUp_, _TimeOut_, _Status_, _Elapsed_
    _PwdIn_ = passinC
    _PwdOut_ = passoutC
    _Cmds_ = cmdsC
    _HangUp_ = hangupL
    _TimeOut_ = timeoutN
    _Status_ = ""
    _Elapsed_ = 0
 ENDIF
 RETURN ONStat                              && We're done

ELSE                                        && Already connected, DCD present

    PUBLIC _PwdIn_, _PwdOut_, _Cmds_, CmdQ        && Declare for __Connect's usage
    PUBLIC _HangUp_, _TimeOut_, _Status_
    _PwdIn_ = passinC
    _PwdOut_ = passoutC
    _Cmds_ = cmdsC
    _HangUp_ = hangupL
    _TimeOut_ = timeoutN
    _Status_ = ""

    DO __Connect                            &&Since we're already connected
ENDIF                                       &&IF DCD

RETURN .T.


****************************** _CALL *******************************
* Purpose:  a) Have modem dial a #  & return to main application.
*           b) upon other modem answering a call, jump to a procedure that does
*              password checking (optional) and starts up any file transfer(s)
*              (also optional).
*
* SYNTAX: _CALL(portN, teleC, passinC, passoutC, timeoutN, hangupL, cmdsC)

* Parameters:
*   portN - specifies COM port  (Will be used in multi port version, COMET-MP)
*   teleC - telephone # to dial
*   passinC - password expected from answering modem (none if passinC = "")
*   passoutC - password sent out to answering modem (none if passoutC = "")
*   timeoutC - # seconds to wait for a connection & valid password receipt
*              before aborting.
*   hangupL  - indicates if auto-disconnect should occur upon completion of cmds
*   cmdsC    - a list of COMET library command(s) delimited by semicolons.
*              ie. cmdsC = "YSEND \ADIR\*.* ; YRECV"  (2 commands)

* Returns:
*   <expL> indicating success (.T.) or failure (.F)
*          NOTE: .T. does not imply that call has been answered, but simply
*                that modem has dialed specified #.
*
*   NOTE: The COM port must be _OPEN() before using this function!

FUNCTION _CALL
PARAMETERS portN, teleC, passinC, passoutC, timeoutN, hangupL, cmdsC
PRIVATE Cmd, Cr
Cr = CHR(13)                                && for hayes modem cmds - end with CR


* Test if a COM port is OPEN
IF TYPE("_Port_") + TYPE("_Error_") # "NC"  && _OPEN() should have declared both
    RETURN .F.                              && If not, can't go on - we,ve failed
ENDIF

IF .NOT. _ISDCD(_Port_) .AND. LEN(teleC) > 0    && No DCD and a tele# supplied ?
   * Attempt to have modem dial tele#
    Cmd = _FLUSH(_Port_)                        && clear buffer
    Cmd = _COMET('OUTPUT AT E1 TD'+ teleC + Cr)  && Echo on, tone dial tele#

    IF .NOT. _FIND(_Port_, 'OK', 2)
        _Error_ = 'CALL() Failed - modem fails to ack command'
        RETURN .F.
    ENDIF

   * Modem should be dialing # at this point

    * Trap for detection of DCD signal = modem answered & connected
    ONStat = _ONDCD(_Port_, '__CONNECT')
    IF ONStat
        PUBLIC _PwdIn_, _PwdOut_, _Cmds_, CmdQ    && Declare for __Connect's usage
        PUBLIC _HangUp_, _TimeOut_, _Status_, _Elapsed_
        _PwdIn_ = passinC
        _PwdOut_ = passoutC
        _Cmds_ = cmdsC
        _HangUp_ = hangupL
        _TimeOut_ = timeoutN
        _Status_ = ""
        _Elapsed_ = 0
    ENDIF
    RETURN ONStat

ELSE                                        && DCD already active, we're connected
        PUBLIC _PwdIn_, _PwdOut_, _Cmds_, CmdQ    && Declare for __Connect's usage
        PUBLIC _HangUp_, _TimeOut_, _Status_
        _PwdIn_ = passinC
        _PwdOut_ = passoutC
        _Cmds_ = cmdsC
        _HangUp_ = hangupL
        _TimeOut_ = timeoutN
        _Status_ = ""

        DO __Connect
ENDIF                                       &&IF .NOT. _ISDCD()
RETURN .T.


****************************** _CLOSE *******************************
* Purpose: To shutdown or close an OPENed COM port.

* SYNTAX: _CLOSE(portN)

* Parameters:
*   portN - specifies which COM port to close  (Will be used in multi port version, COMET-MP)

* Returns:
*   <expL> indicating success (.T.) or failure (.F)

FUNCTION _CLOSE
PARAMETERS portN
PRIVATE portC, Cmd1, Cmd2

portC = LTRIM(STR(portN,1))                   && Convert portN to <expC>
Cmd1 = _COMET('FLUSH ' + portC)            && 1st, FLUSH port
Cmd2 = _COMET('CLOSE ' + portC)            && Now, CLOSE COM port

RETURN IIF('FLUSH' $ Cmd1 .AND. 'CLOSE' $ Cmd2, .T., .F.)

****************************** _COMET *******************************
* Purpose: To execute a COMET command using CALL statement

* SYNTAX: _COMET(Command)

* Parameters:
*   Command - COMET formatted command <expC>

* Returns:
*   Command (the variable returned by COMET) or null string (failed)  <expC>

FUNCTION _COMET
PARAMETERS Command
PRIVATE OrigCmd

OrigCmd = Command                           && Save original Command
CALL COMET WITH Command                     && Perform COMET command
IF LEFT(OrigCmd,2) <> LEFT(Command,2)       && Test for COMET error
    PUBLIC FailCmd
    FailCmd = Command                       && Save COMET returned cmd/error
    Command = ''                            && Return null string to flg error
ENDIF

RETURN Command


****************************** _DISCON *****************************
* Purpose: To disconnect a Hayes style modem

* SYNTAX: _DISCON(portN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)

* Returns:
*   <expL> indicating success (.T.) or failure (.F) based on state of DCD signal

FUNCTION _DISCON
PARAMETERS portN
PRIVATE Cmd, Rtn

Cmd  = _FIND(_Port_, '~@~', 2)             && Delay 1-2 secs
Cmd = _OUTPUT(_Port_, '+++')              && Send online disconnect command
Cmd = _FIND(_Port_, '~@~', 2)
Cmd = _OUTPUT(_Port_, 'ATH0' + CHR(10) + CHR(13) )  && Go on hook
Cmd = _SETDTR(_Port_, .F.)                && Disable DTR - another method of disconnecting
Cmd = _FIND(_Port_, '~@~', 2)
Cmd = _SETDTR(_Port_, .T.)                && Enable DTR
RETURN (.NOT. _ISDCD(_Port_) )


****************************** _FIND *******************************
* Purpose: To search COMET's receive buffer for a character string

* SYNTAX: _FIND(portN, stringC, timeoutN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   stringC - character(s) to search for  <expC>
*   timeoutN - maximum # seconds to search before aborting    <expN)
*               0 = wait till found (ie. forever = DANGEROUS)
*              -1 = return immediately whether found or not
*              NOTE: Due to the use of dBASE's TIME() function which only
*                    has a resolution of 1 second, the actual timeout can
*                    be upto 1 second less than specified timeoutN.
*                    If your database language supports SECONDS() function,
*                    you can virtually eliminate this inaccuracy.

* Returns:
*   <expL> indicating found (.T.) or not found (.F)

FUNCTION _FIND
PARAMETERS portN, stringC, timeoutN
PRIVATE Cmd, LasTime

Cmd = _COMET('FIND ' + stringC)             && Initial search for stringC
LasTime = TIME()

* Loop to FIND stringC until found or time has expired
DO WHILE AT('FIND+', Cmd) # 1 .AND. timeoutN # 0
    Cmd = _COMET(Cmd)                      && FIND

   * Timer logic
    CurrTime = TIME()    
    IF LasTime <> CurrTime                  && Another second gone by ?
        LasTime = CurrTime                  && Yes, update & count down
        timeoutN = IIF(timeoutN > 0, timeoutN - 1, timeoutN)
    ENDIF
ENDDO

RETURN IIF(AT('FIND+', Cmd) # 1, .F., .T.)

****************************** _FLUSH *******************************
* Purpose: To clear/discard any COM data in COMET's receive buffer

* SYNTAX: _FLUSH(portN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)

* Returns:
*   <expL> indicating success (.T.) or failure (.F)

FUNCTION _FLUSH
PARAMETERS portN
PRIVATE Cmd, portC

portC = LTRIM(STR(portN))                   && Convert portN to <expC>
Cmd  = _COMET('FLUSH ' + portC)            && FLUSH COM port

RETURN IIF('FLUSH' $ Cmd, .T., .F.)

****************************** _INPCHR *******************************
* Purpose: To read 1 characters from COM port's receive buffer

* SYNTAX: _INPCHR(portN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)

* Returns:
*   <expC> - COM port data character

FUNCTION _INPCHR
PARAMETERS portN                    && portN - not used by single port COMET
PRIVATE COMChar

COMChar = _INPSTR(portN, 1)         && Go read 1 char

RETURN COMChar


****************************** _INPSTR *******************************
* Purpose: To read string of characters from a COM port's receive buffer

* SYNTAX: _INPSTR(portN, charsN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   charsN - # chars to read from buffer

* Returns:
*   <expC> - COM port data

FUNCTION _INPSTR
PARAMETERS portN, charsN                    && portN - not used by single port COMET
PRIVATE InpCmd, Cnt, COMData

* CHR(255) is termination char (we don't want one, so specify an unlikely CHR
* Add 4 extra SPACEs to compensate for returned byte count
InpCmd = _COMET('INPUT ' + CHR(255) + SPACE(charsN + 4) )

COMData = ''                                && Init data

* Test if INPUT command was rejected (probably cause file xfer other than ARECV in progress)
IF LEN(InpCmd) > 0                          && Not rejected
    Cnt = VAL(SUBSTR(InpCmd,8,4))           && Get count of # chars read
    COMData = IIF(Cnt > 0, SUBSTR(InpCmd,12,Cnt), '')
ENDIF


RETURN COMData


****************************** _INPWAIT *******************************
* Purpose: To read character(s) from COM port, waiting specified time if needed

* SYNTAX: _INPWAIT(portN, charsN, timeoutN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   charsN - # chars to read
*   timeoutN - # seconds to wait if specified # chars not yet in buffer
*              0 = don't wait           -1 = wait forever (DANGEROUS)

* Returns:
*   <expC> - COM port data character(s)

FUNCTION _INPWAIT
PARAMETERS portN, charsN, timeoutN
PRIVATE COMData, CurrTime, LasTime

COMData = _INPSTR(portN, charsN)            && Let's try to read it in one shot
LasTime = TIME()

Need = charsN - LEN(COMData)
* If we didn't read requested # chars and timeout specified, keep reading
DO WHILE Need > 0 .AND. timeoutN # 0
    COMData = COMData + _INPSTR(portN, Need)
    Need = charsN - LEN(COMData)
 * Timer logic
    CurrTime = TIME()    
    IF LasTime <> CurrTime                  && Another second gone by ?
        LasTime = CurrTime                  && Yes, update & count down
        timeoutN = IIF(timeoutN > 0, timeoutN - 1, timeoutN)
    ENDIF
ENDDO

RETURN COMData


****************************** _ISCTS *******************************
* Purpose: To obtain the state of COM port's Clear To Send(CTS) input signal

* SYNTAX: _ISCTS(portN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)

* Returns:
*   <expL> - indicating signal is active (.T.) or inactive (.F.)

FUNCTION _ISCTS
PARAMETERS portN
PRIVATE Cmd

Cmd = _COMET('MSTAT ' + SPACE(20) )

RETURN IIF('+CTS' $ Cmd, .T., .F.)



****************************** _ISDCD *******************************
* Purpose: To obtain the state of COM port's Data Carrier Detect (DCD) input signal

* SYNTAX: _ISDCD(portN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)

* Returns:
*   <expL> - indicating signal is active (.T.) or inactive (.F.)

FUNCTION _ISDCD
PARAMETERS portN
PRIVATE Cmd

Cmd = _COMET('MSTAT ' + SPACE(20) )         && Get Modem STATus

RETURN IIF('+DCD' $ Cmd, .T., .F.)



****************************** _ISDSR *******************************
* Purpose: To obtain the state of COM port's Data Set Ready (DSR) input signal

* SYNTAX: _ISDSR(portN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)

* Returns:
*   <expL> - indicating signal is active (.T.) or inactive (.F.)

FUNCTION _ISDSR
PARAMETERS portN
PRIVATE Cmd

Cmd = _COMET('MSTAT ' + SPACE(20) )         && Get Modem STATus

RETURN IIF('+DSR' $ Cmd, .T., .F.)


****************************** _ISRI *******************************
* Purpose: To obtain the state of COM port's Ring Indicator (RI) input signal

* SYNTAX: _ISRI(portN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)

* Returns:
*   <expL> - indicating signal is active (.T.) or inactive (.F.)

FUNCTION _ISRI
PARAMETERS portN
PRIVATE Cmd

Cmd = _COMET('MSTAT ' + SPACE(20) )         && Get Modem STATus

RETURN IIF('+RI' $ Cmd, .T., .F.)


****************************** _MODEM *******************************
* Purpose: Attempts to locate and initialize a Hayes modem at either COM1 or 2
*          at 2400 or 1200 baud. (2400 attempted first)
*          This function leaves COM1 or 2 OPENed if a modem is found.
*          Easily modified to test additional ports/baud rates by modifying
*          ComPort() & Baud() arrays.
*          NOTE: This function always opens a COM port at 8 databits, No parity,
*                1 stop bit and No flow control.  You could pass these parameters
*                to this function by adding a PARAMETERS statement and replacing
*                these fixed values within this function with param var's names.
*                OR, since this function leaves the COM port open if a modem is
*                found, you can simply issue an _OPEN(_Port_, ...) command to
*                parity and data bits.
* SYNTAX: _MODEM()

* Parameters:  NONE

* Returns:
*   <expL> - indicating whether a modem is found (.T.) or not (.F.)
*            Also, if modem is found, a PUBLIC variable named _Port_ will contain
*            the COM port # where modem was found.

FUNCTION _MODEM
PRIVATE FndOK, Cr, Cmd, NumPorts, NumBauds, Port, Baud, TryPort, TryBaud

* Here's where you can set which ports and baud rates to try.
******************************************************************************
NumPorts = 2                          && We're going to test 2 ports (COM1-2)
NumBauds = 2                          && And try 2 different baud rates (1200, 2400)
Declare ComPort(NumPorts), Baud(NumBauds)
ComPort(1) = 1                        && 1st port to try = COM1
ComPort(2) = 2                        && 2nd is COM2
Baud(1)    = 2400                     && Try 2400 1st for any COM port
Baud(2)    = 1200                     && 2nd, try 1200
******************************************************************************

Cr = CHR(13)
amp = CHR(38)                         && ampersand character

* Hayes commands we'll use:
*   E1 = Echo on
*   V1 = Verbal result codes
*   X1 = eXtended result codes (ie. CONNECT msg will include baud rate)
*   H0 = go on Hook (ie. hang up)
*   Q0 = Quiet mode off
*   &C1 = modem's DCD output signal to reflect modem Carrier state
*   &D1 = modem's DTR input signal will cause modem disconnect if DTR from PC goes low

FndOK = .F.                     && Will get set .T. if modem detected
STORE 1 TO TryPort, TryBaud         && Pntrs into ComPort() & Baud()

DO WHILE TryPort <= NumPorts .AND. .NOT. FndOK    && COM Port Loop
    Port = ComPort(TryPort)         && Current port# to try
    BRate = Baud(1)                 && Current baud = 1st
    Cmd = _OPEN(Port, BRate,'N',8,1,'N')       && Open a port

    IF _ISDSR(Port) .AND. .NOT. _ISRI(Port)    && If modem at port: +DSR & -RI
    DO WHILE TryBaud <= NumBauds .AND. .NOT. FndOK    && Baud loop
        BRate = Baud(TryBaud)                && Current baud to try
        Cmd = _OPEN(Port, BRate,'N',8,1,'N')       && Open a port
        Cmd = _OUTPUT(Port, 'ATZ' + Cr)    && Issue HAYES modem RESET comand
        Cmd = _FIND(Port, 'OK', 1)       && Modem should respond with OK message
        Delay1 = _FIND(Port, '~!~', 1)     && Wait a second for modem to process cmd
        Cmd = _OUTPUT(Port,'ATQ0E1V1X1H0'+Cr) && Send HAYES MODEM commnds defined above
        FndOK = _FIND(Port, 'OK', 1)       && Modem should respond with OK message
        Delay1 = _FIND(Port, '~!~', 1)     && Wait a second for modem to process cmd
        TryBaud = TryBaud + 1
    ENDDO
    ENDIF

    TryPort = TryPort + 1               && Setup for next port# to try
    TryBaud = 1
    IF .NOT. FndOK                      && Close port if modem not found
        Cmd = _CLOSE(Port)
    ENDIF
ENDDO


* If we've located a modem at COM port, let's issue the &C1 and &D1 commands
* just in case this brand modem recognizes these commands as valid.  If not,
* no real harm done as modem will just ignore them or output an error msg.
IF FndOK
    Cmd = _OUTPUT(_Port_, 'AT' + amp+'C1' + amp+'D1' + Cr)
    OK  = _FIND(Port, 'OK', 1)
    Delay1 = _FIND(Port, '~!~', 1)      && Wait 1 sec
ENDIF
    
RETURN FndOk


****************************** _ONDCD *******************************
* Purpose: To jump to a specified procedure upon COM port's DCD signal changing.
           NOTE: Will jump to proc when DCD goes from  ON->OFF  or  OFF->ON !

* SYNTAX: _ONDCD(portN, dcdprocC, keyascN, keyscanN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   dcdprocC - Procedure name to jump to.
*              NOTE: specifying a null string will disable ONDCD events.

* Returns:
*   <expL> - indicating success or failure

FUNCTION _ONDCD
PARAMETERS portN, procnameC
PRIVATE Cmd

IF LEN(procnameC) > 0                       && Enable DCD event trapping ?

*** Choose appropriate statement below for your language:
*   SET ESCAPE ON                          && dBASE & FoxBase
*   ON ESCAPE DO &procnameC                && dBASE, FoxBase

    ON KEY LABEL Alt-F8 DO &procnameC      && FoxPro -  Alt-F8 for ONDCD's

*   Cmd = _COMET('ONDCD 27,1')              && dBASE & FoxBase
    Cmd = _COMET('ONDCD 0,111')             && FoxPro

ELSE                                        && Disable ONDCD
    Cmd = _COMET('ONDCD ')
*    ON ESCAPE                               && dBASE & FoxBase - disable
    ON KEY LABEL Alt-F8                      && FoxPro - disable  "     "
ENDIF

RETURN  IIF('ONDCD' $ Cmd, .T., .F.)



****************************** _ONDONE *******************************
* Purpose: To jump to a specified procedure upon a file transfer completion or failure

* SYNTAX: _ONDONE(portN, procnameC)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   procname - Procedure name to jump to.
*              NOTE: specifying a null string will disable ONDONE events.

* Returns:
*   <expL> - indicating success or failure

FUNCTION _ONDONE
PARAMETERS portN, procnameC
PRIVATE Cmd

IF LEN(procnameC) > 0                       && Enable file xfer done event trapping ?

*** Choose appropriate statement below for your language:
*   SET ESCAPE ON                          && dBASE & FoxBase
*   ON ESCAPE DO &procnameC                && dBASE, FoxBase

    ON KEY LABEL Alt-F9 DO &procnameC      && FoxPro -  Alt-F9 for ONDONE's

*   Cmd = _COMET('ONDONE 27,1')              && dBASE & FoxBase
    Cmd = _COMET('ONDONE 0,112')             && FoxPro

ELSE                                        && Disable ONDCD
    Cmd = _COMET('ONDONE ')
*    ON ESCAPE                               && dBASE & FoxBase - disable
    ON KEY LABEL Alt-F9                      && FoxPro - disable  "     "
ENDIF

RETURN  IIF('ONDONE' $ Cmd, .T., .F.)



****************************** _ONTIME *******************************
* Purpose: To jump to a specified procedure every n seconds

* SYNTAX: _ONTIME(portN, secondsN, procnameC)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   secondsN - the desired time interval in seconds between events
*              NOTE: Specifying secondsN = 0 will disable ONTIME too.
*   procnameC - Procedure name to jump to.
*              NOTE: specifying a null string will disable ONTIME events.

* Returns:
*   <expL> - indicating success or failure

FUNCTION _ONTIME
PARAMETERS portN, secondsN, procnameC
PRIVATE Cmd, secondsC


IF LEN(procnameC) > 0 .AND. secondsN > 0    && Enable ONTIME events

	secondsC = LTRIM(STR(secondsN,5))
*** Choose appropriate statement below for your language:

*   SET ESCAPE ON                          && dBASE & FoxBase
*   ON ESCAPE DO &procnameC                && dBASE, FoxBase

    ON KEY LABEL Alt-F7 DO &procnameC      && FoxPro -  Alt-F7 for ONDONE's

*   Cmd = _COMET('ONTIME ' + secondsC + ',27,1')              && dBASE & FoxBase
    Cmd = _COMET('ONTIME ' + secondsC + ',0,110')             && FoxPro

ELSE                                        && Disable ONDCD
    Cmd = _COMET('ONTIME ')
*    ON ESCAPE                               && dBASE & FoxBase - disable
    ON KEY LABEL Alt-F7                      && FoxPro - disable  "     "
ENDIF

RETURN  IIF('ONTIME' $ Cmd, .T., .F.)


****************************** _OPEN *******************************
* Purpose: To initialize a COM port for use.

* SYNTAX: _OPEN(portN, baudN, parityC, dataN, stopN, flowC)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   baudN - baud rate desired (300 - 38,400)
*   parityC - 'E', 'O' or 'N'   for Even, Odd or None
*   dataN   - 7 or 8 data bits
*   stopN   - 1 or 2 stop bit(s)
*   flowC   - 'N', 'R' or 'X'  for None, Rts/cts or Xon/xoff

* Returns:
*   <expL> - indicating success or failure

* NOTE: If OPEN is successful, this function will declare PUBLIC _Port_,
*       the numeric value of the COM port opened.  This variable can then
*       be used for all other functions requiring a port# parameter.
*       Also, _Error_ will be declared for returning error messages from
*       most of these functions.

FUNCTION _OPEN
PARAMETERS portN, baudN, parityC, dataN, stopN, flowC
PRIVATE Cmd, portC, baudC, dataC, stopC, Rtn


portC = LTRIM(STR(portN,1))
baudC = LTRIM(STR(baudN,5))
dataC = LTRIM(STR(dataN,1))
stopC = LTRIM(STR(stopN,1))

Cmd = _COMET('OPEN COM' + portC + ':' + baudC + ',';
+ parityC + ',' + dataC + ',' + stopC + ',' + flowC)

IF AT('OPEN ', Cmd) > 0                     && Was OPEN successful ?
    Rtn = .T.                               && Yep, so: return true
    PUBLIC _Port_, _Baud_,_Error_           && Declare a public port# var
    _Port_ = portN
    _Baud_ = baudN
    _Error_ = ""                            && and an var for passing error msgs
ELSE
    Rtn = .F.                               && No, return false
ENDIF

RETURN Rtn



****************************** _OUTPUT *******************************
* Purpose: To output character(s) to a COM port

* SYNTAX: _OUTPUT(portN, stringC)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   stringC - character(s) to send out the COM port

* Returns:
*   <expL> - indicating success or failure

FUNCTION _OUTPUT
PARAMETERS portN, stringC
PRIVATE Cmd

Cmd = _COMET('OUTPUT ' + stringC)

RETURN IIF(AT('OUTPUT', Cmd) = 1, .T., .F.)


****************************** _SETDTR *******************************
* Purpose: To set COM port's Data Terminal Ready (DTR) output signal on or off

* SYNTAX: _SETDTR(portN, OnOff)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   OnOff - Indicate whether to set DTR signal on (.T) or off (.F.)  <expL>

* Returns:
*   <expL> indicating success (.T.) or failure (.F)

FUNCTION _SETDTR
PARAMETERS portN, OnOff
PRIVATE Cmd

Cmd = _COMET('DTR ' + IIF(OnOff, '1', '0'))               && Set DTR as requested

RETURN IIF('DTR' $ Cmd, .T., .F.)



****************************** _SETRTS *******************************
* Purpose: To set COM port's Request To Send (RTS) output signal on or off.

* SYNTAX: _SETRTS(portN, OnOff)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)
*   OnOff - Indicate whether to set RTS signal on (.T) or off (.F.)  <expL>

* Returns:
*   <expL> indicating success (.T.) or failure (.F)

FUNCTION _SETRTS
PARAMETERS portN, OnOff
PRIVATE Cmd

Cmd = _COMET('RTS ' + IIF(OnOff, '1', '0')) && Set RTS as requested

RETURN IIF('RTS' $ Cmd, .T., .F.)


****************************** _RXCNT *******************************
* Purpose: To find out how many characters are present in COM buffer

* SYNTAX: _RXCNT(portN)

* Parameters:
*   portN - COM port#  <expN>  (Will be used in multi port version, COMET-MP)

* Returns:
*   <expN> indicating # of chars in buffer

FUNCTION _RXCNT
PARAMETERS portN
PRIVATE Cmd, Cnt

Cmd = _COMET('RXCNT     ')
Cnt = IIF('RXCNT' $ Cmd, VAL(SUBSTR(Cmd,7)), -1)

RETURN Cnt

***************************** NOTE *******************************************
* The following routines are event trapping related & used by the
* _CALL() & _AUTOANS() routines.  DO NOT use them as general purpose
* procedures.
***************************** NOTE *******************************************

************************* __Connect   (Internal use ONLY) **********************
* Purpose: This routine should execute automatically when a modem answers
*          and connects with another modem.  This routine:
*               1) Adjusts COMET's baud rate to match baud rate reported in
*                  modem's  "CONNECT ####" message.
*               2) Exchange and verify passwords if requested in _CALL() or
*                  _AUTOANS() UDFs.
*               3) Begin the 1st requested COMET ?SEND or ?RECV command
*                  requested in _CALL() or _AUTOANS() UDFs.
*               4) Using an ONDONE event trap, have __CmdDone routine process
*                  any additional ?SEND or ?RECV commands.


PROCEDURE __Connect

PRIVATE Cmd, Fnd, Cmd1, Cmd1End, CrLf, NewBaud, BaudOK, ConnFnd, baudC
CrLf = CHR(10) + CHR(13)

Cmd = _ONDCD(_Port_, '')                && disable DCD trapping

* Now check that this connection's baud rate = our OPENed _Baud_ rate.
baudC = LTRIM(STR(_Baud_,5))
BaudOK = _FIND(_Port_, "CONNECT " + baudC, 0)        && Does CONNECT message's baud rate = COMET's OPENed rate ?
IF .NOT. BaudOK                         && Either a different connect rate or no message
    ConnFnd = _FIND(_Port_, 'CONNECT ', 2) && Look for "CONNECT " msg from modem
    IF .NOT. ConnFnd                    && Bad news, no CONNECT msg at all
         _Error_ = DTOC(DATE()) + '  ' + TIME() + '  ' + 'ERROR: NO "CONNECT ####" MESSAGE FROM MODEM' + CrLf
         Cmd =  _DISCON(_Port_)         && Disconnect from line
         Cmd = _ONDCD(_Port_, '__Connect') && Back to waiting for a call
         RETURN                        && Out of here! (go back to main appl.)
    ENDIF

    * Got a CONNECT message, let's get the reported "CONNECT ####" baud rate
	Cmd = _INPSTR(_Port_,8)						&& Let's get CONNECT from buffer
    NewBaud = LTRIM(TRIM(_INPSTR(_Port_,4)))    && Get CONNECT "####" baud rate
    IF LEN(NewBaud) < 3 .OR. .NOT. NewBaud $ '300 1200 2400 4800 9600'  && Add add'l rates if modem supports
         _Error_ = DTOC(DATE()) + '  ' + TIME() + '  ' + 'ERROR: INVALID "CONNECT ' + NewBaud + '" BAUD RATE' + CrLf
         Cmd = _DISCON(_Port_)                && Disconnect from line
         Cmd = _ONDCD(_Port_, '__Connect') && Back to waiting for a call
         RETURN                        && Out of here! (go back to main appl.)
    ENDIF

    * New valid baud rate determined, now OPEN with new rate
    NewRateN = VAL(NewBaud)
    Cmd = _OPEN(_Port_, NewRateN, 'N', 8, 1, 'N')

ENDIF                                   && IF BaudOK


Start = TIME()                                 && This is a way to wait 1 second
DO WHILE TIME() = Start                        && for phone line to stabalize
ENDDO
* Exchange passwords if requested
* Send 3 copies of our password, if any       && Send 3 copies in case noisy connection
IF LEN(_PwdOut_) > 0
    Cmd = _OUTPUT(_Port_, REPLICATE(_PwdOut_,3))
ENDIF

* Check incoming password, if any
IF LEN(_PwdIn_) > 0
    Fnd = _FIND(_Port_, UPPER(_PwdIn_), _TimeOut_)
ELSE                                    && Incoming password check not requested
    Fnd = .T.
ENDIF

IF Fnd
* Make a copy of _CALL() or _AUTOANS() command(s)
    CmdQ = _Cmds_ + CHR(59)                   && Make last cmd end with semi-colon
* Extract 1st command
    Cmd1End = AT(CHR(59), CmdQ)                   && Look for 1st
    Cmd1 = LTRIM(RTRIM(LEFT(CmdQ, Cmd1End-1)))  && Extract 1st COMET command
    CmdQ = IIF(LEN(CmdQ) > Cmd1End, SUBSTR(CmdQ, Cmd1End+1), '')  && Remove it from Queue
* Get COMET working on 1st file xfer command requested
    Cmd = _COMET(Cmd1)
* Set ONDONE event branch to __CmdDone for processing of subsequent commands
    Cmd = _ONDONE(_Port_, '__CmdDone')

ELSE                        && Password not found, abort
    _Error_ = DTOC(DATE()) + '  ' + TIME() + '  ' + 'ERROR: PASSWORD NOT RECEIVED'
    Cmd = _Discon(_Port_)
    Cmd = _ONDCD(_Port_, '__Connect') && Back to waiting for a call
ENDIF                                   && IF Fnd
RETURN                                  && Return back to main application


********************** __CmdDone  (Internal use only) *************************

*Purpose: To execute next COMET command (if any) from _AUTOANS() or _CALL().
*         Also, if no more commands to process, disconnect if requested.

PROCEDURE __CmdDone
PRIVATE Cmd, NxtCmd, NewLogLn, OldLog, CrLf
CrLf = CHR(13) + CHR(10)

* Save the results of last command
_Status_ = _COMET('FCHK ' + SPACE(100))  && Get last COMET command's status

* Test for loss of modem connection (ie. DCD)
IF .NOT. _ISDCD(_Port_)
    _Error_ = TRIM(_Status_) + ' [CARRIER LOST]'
    Cmd = _COMET('FLUSH ')
    Cmd = _DisCon(_Port_)
    Cmd = _COMET('FLUSH ')
    Cmd = _ONDCD(_Port_, '__Connect') && Back to waiting for a call
    RETURN                        && Out of here! (go back to main appl.)

ENDIF

IF .NOT. 'ACTIVE' $ _Status_

  IF .NOT. 'COMPLETE' $ _Status_              && Test for a failure
    _Error_ = _Status_
    Cmd = _DISCON(_Port_)                           && If failed, disconnect
    Cmd = _ONDCD(_Port_, '__Connect') && Back to waiting for a call
    RETURN                        && Out of here! (go back to main appl.)
  ENDIF

* A _CALL() or _AUTOANS() command has successfully completed
* Now see if another command should be performed
  NxtEnd = AT(CHR(59), CmdQ)                && Look for SEMICOLON
  IF NxtEnd > 0                               && More commands ?
    * Extract next command
      NxtCmd = LTRIM(RTRIM(LEFT(CmdQ, NxtEnd-1)))
      CmdQ = IIF(LEN(CmdQ) > NxtEnd, SUBSTR(CmdQ, NxtEnd+1), '')  && Remove it from list
    * Execute it
      Cmd = _COMET(NxtCmd)
      Cmd = _ONDONE(_Port_, '__CmdDone')

* No more commands to do
  ELSE                                        && All done, no more cmds for COMET
      Cmd = _ONDONE(_Port_, '')               && Disable file xfer done trapping

* See if we're supposed to hangup when done
      IF _HangUp_                             && Did _AUTOANS() or _CALL() specify disconnect ?
         Cmd = _DISCON(_Port_)
      ENDIF

      Cmd = _ONDCD(_Port_, '__Connect') && Back to waiting for a call
  ENDIF

 ENDIF                                           && COMET ACTIVE ?
RETURN


